﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectMachine
{
    /// <summary>
    /// Класс, отвечающий за прорисовку и перемещение объекта-сущности
    /// </summary>
    public class DrawningMachine
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityMachine Machine { get; protected set; }
        /// <summary>
        /// Левая координата отрисовки машины
        /// </summary>
        protected float _startPosX;
        /// <summary>
        /// Верхняя кооридната отрисовки машины
        /// </summary>
        protected float _startPosY;
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private int? _pictureWidth = null;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private int? _pictureHeight = null;
        /// <summary>
        /// Ширина отрисовки автомобиля
        /// </summary>
        private readonly int _machineWidth = 90;
        /// <summary>
        /// Высота отрисовки автомобиля
        /// </summary>
        private readonly int _machineHeight = 40;
        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес автомобиля</param>
        /// <param name="bodyColor">Цвет кузова</param>
        public DrawningMachine(int speed, float weight, Color bodyColor)
        {
            Machine = new EntityMachine(speed, weight, bodyColor);
        }
        /// <summary>
        /// Установка позиции машины
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        protected DrawningMachine(int speed, float weight, Color bodyColor, int machineWidth, int machineHeight) :
            this(speed, weight, bodyColor)
        {
            _machineWidth = machineWidth;
            _machineHeight = machineHeight;
        }

        public void SetPosition(int x, int y, int width, int height)
        {
            if (x < 0 || y < 0 || x + _machineWidth > width || y + _machineHeight > height)
            {
                return;
            }
            _startPosX = x;
            _startPosY = y;
            _pictureWidth = width;
            _pictureHeight = height;
        }
        /// <summary>
        /// Изменение направления пермещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(Direction direction)
        {
            if (!_pictureWidth.HasValue || !_pictureHeight.HasValue)
            {
                return;
            }
            switch (direction)
            {
                // вправо
                case Direction.Right:
                    if (_startPosX + _machineWidth + Machine.Step < _pictureWidth)
                    {
                        _startPosX += Machine.Step;
                    }
                    break;
                //влево
                case Direction.Left:
                    if (_startPosX - Machine.Step > 0)
                    {
                        _startPosX -= Machine.Step;
                    }
                    break;
                //вверх
                case Direction.Up:
                    if (_startPosY - Machine.Step > 0)
                    {
                        _startPosY -= Machine.Step;
                    }
                    break;
                //вниз
                case Direction.Down:
                    if (_startPosY + _machineHeight + Machine.Step < _pictureHeight)
                    {
                        _startPosY += Machine.Step;
                    }
                    break;
            }
        }
        /// <summary>
        /// Отрисовка машины
        /// </summary>
        /// <param name="g"></param>
        public virtual void DrawTransport(Graphics g)
        {
            if (_startPosX < 0 || _startPosY < 0
                || !_pictureHeight.HasValue || !_pictureWidth.HasValue)
            {
                return;
            }
            Pen pen = new(Color.Black);
            Brush brBodyColor = new SolidBrush(Machine.BodyColor);

            // Граница гусениц
            g.DrawEllipse(pen, _startPosX, _startPosY + 11, 90, 22);

            // Кузов
            g.FillRectangle(brBodyColor, _startPosX + 25, _startPosY, 40, 10);
            g.FillRectangle(brBodyColor, _startPosX, _startPosY + 10, 90, 10);

            // Катки
            g.DrawEllipse(pen, _startPosX + 5, _startPosY + 19, 10, 10);
            g.DrawEllipse(pen, _startPosX + 76, _startPosY + 19, 10, 10);
            g.DrawEllipse(pen, _startPosX + 19, _startPosY + 25, 6, 6);
            g.DrawEllipse(pen, _startPosX + 34, _startPosY + 25, 6, 6);
            g.DrawEllipse(pen, _startPosX + 49, _startPosY + 25, 6, 6);
            g.DrawEllipse(pen, _startPosX + 64, _startPosY + 25, 6, 6);
        }
        /// <summary>
        /// Смена границ формы отрисовки
        /// </summary>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public void ChangeBorders(int width, int height)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            if (_pictureWidth <= _machineWidth || _pictureHeight <= _machineHeight)
            {
                _pictureWidth = null;
                _pictureHeight = null;
                return;
            }
            if (_startPosX + _machineWidth > _pictureWidth)
            {
                _startPosX = _pictureWidth.Value - _machineWidth;
            }
            if (_startPosY + _machineHeight > _pictureHeight)
            {
                _startPosY = _pictureHeight.Value - _machineHeight;
            }
        }
        /// <summary>
        /// Получение текущей позиции объекта
        /// </summary>
        /// <returns></returns>
        public (float Left, float Right, float Top, float Bottom) GetCurrentPosition()
        {
            return (_startPosX, _startPosX + _machineWidth, _startPosY, _startPosY + _machineHeight);
        }
    }
}
